"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const queryExecutor_1 = __importDefault(require("../../../utils/queryExecutor"));
const sfpowerkit_1 = require("../../../sfpowerkit");
const chunkArray_1 = require("../../../utils/chunkArray");
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const CODECOVAGG_QUERY = `SELECT Id FROM ApexCodeCoverageAggregate`;
const APEXTESTRESULT_QUERY = `SELECT Id FROM ApexTestResult`;
class Cleartestresult extends command_1.SfdxCommand {
    async run() {
        sfpowerkit_1.SFPowerkit.setLogLevel("Info", this.flags.json);
        await this.org.refreshAuth();
        const conn = this.org.getConnection();
        this.ux.startSpinner(`Clearing Test results`);
        let queryUtil = new queryExecutor_1.default(conn);
        let codeCovAgg = await queryUtil.executeQuery(CODECOVAGG_QUERY, true);
        await this.deleteRecords(conn, "ApexCodeCoverageAggregate", codeCovAgg);
        let testResults = await queryUtil.executeQuery(APEXTESTRESULT_QUERY, true);
        await this.deleteRecords(conn, "ApexTestResult", testResults);
        this.ux.stopSpinner();
        sfpowerkit_1.SFPowerkit.log(`Test results cleared in ${this.org.getUsername()} successfully.`, sfpowerkit_1.LoggerLevel.INFO);
        return true;
    }
    async deleteRecords(conn, objectType, records) {
        if (records && records.length > 0) {
            let idsList = records.map((elem) => elem.Id);
            let errors = [];
            for (let idsTodelete of chunkArray_1.chunkArray(2000, idsList)) {
                const deleteResults = await conn.tooling.destroy(objectType, idsTodelete);
                deleteResults.forEach((elem) => {
                    if (!elem.success) {
                        errors = errors.concat(elem.errors);
                    }
                });
            }
            if (errors.length > 0) {
                throw new core_1.SfdxError(JSON.stringify(errors));
            }
        }
    }
}
exports.default = Cleartestresult;
Cleartestresult.description = `This command helps to clear any test results and code coverage in the org to get fresh and enhanced coverage everytime`;
Cleartestresult.examples = [
    `$ sfdx sfpowerkit:org:cleartestresult -u myOrg@example.com`,
];
// Comment this out if your command does not require an org username
Cleartestresult.requiresUsername = true;
//# sourceMappingURL=cleartestresult.js.map