"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const relaxIPRangeImpl_1 = __importDefault(require("../../../impl/org/relaxIPRangeImpl"));
// tslint:disable-next-line:ordered-imports
var path = require("path");
const sfpowerkit_1 = require("../../../sfpowerkit");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "org_relaxiprange");
class Relaxiprange extends command_1.SfdxCommand {
    async run() {
        sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
        //validate only one param is passed
        if (!this.flags.range && !this.flags.all && !this.flags.none) {
            throw new core_1.SfdxError(`Required input is missing. you must pass anyone of the flag -r (or) --all (or) --none`);
        }
        else if ((this.flags.range && this.flags.all) ||
            (this.flags.range && this.flags.none) ||
            (this.flags.none && this.flags.all)) {
            throw new core_1.SfdxError(`Too many inputs found, you must pass only one param -r (or) --all (or) --none`);
        }
        let ipRangeToSet = [];
        if (this.flags.range) {
            ipRangeToSet = this.flags.range.map(function (element) {
                let range = element.split("-");
                return { start: range[0], end: range[1] };
            });
        }
        return await relaxIPRangeImpl_1.default.setIp(this.org.getConnection(), this.org.getUsername(), ipRangeToSet, this.flags.all, this.flags.none);
    }
}
exports.default = Relaxiprange;
Relaxiprange.description = messages.getMessage("commandDescription");
Relaxiprange.examples = [
    `sfdx sfpowerkit:org:relaxiprange -u sandbox -r "122.0.0.0-122.255.255.255,49.0.0.0-49.255.255.255"`,
    `sfdx sfpowerkit:org:relaxiprange -u sandbox --all`,
    `sfdx sfpowerkit:org:relaxiprange -u sandbox --none`
];
Relaxiprange.flagsConfig = {
    range: command_1.flags.array({
        required: false,
        char: "r",
        description: messages.getMessage("rangeFlagDescription")
    }),
    all: command_1.flags.boolean({
        description: messages.getMessage("allDescription"),
        required: false
    }),
    none: command_1.flags.boolean({
        description: messages.getMessage("noneDescription"),
        required: false
    }),
    loglevel: command_1.flags.enum({
        description: "logging level for this command invocation",
        default: "info",
        required: false,
        options: [
            "trace",
            "debug",
            "info",
            "warn",
            "error",
            "fatal",
            "TRACE",
            "DEBUG",
            "INFO",
            "WARN",
            "ERROR",
            "FATAL"
        ]
    })
};
// Comment this out if your command does not require an org username
Relaxiprange.requiresUsername = true;
//# sourceMappingURL=relaxiprange.js.map