"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
let request = require("request-promise-native");
const core_1 = require("@salesforce/core");
const sfpowerkit_1 = require("../../../../sfpowerkit");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "sandbox_create");
class Create extends command_1.SfdxCommand {
    async run() {
        sfpowerkit_1.SFPowerkit.setLogLevel("INFO", false);
        await this.hubOrg.refreshAuth();
        const conn = this.hubOrg.getConnection();
        this.flags.apiversion =
            this.flags.apiversion || (await conn.retrieveMaxApiVersion());
        const uri = `${conn.instanceUrl}/services/data/v${this.flags.apiversion}/tooling/sobjects/SandboxInfo/`;
        var result;
        if (this.flags.clonefrom) {
            const sourceSandboxId = await this.getSandboxId(conn, this.flags.clonefrom);
            result = await request({
                method: "post",
                uri,
                headers: {
                    Authorization: `Bearer ${conn.accessToken}`
                },
                body: {
                    AutoActivate: "true",
                    SandboxName: `${this.flags.name}`,
                    Description: `${this.flags.description}`,
                    ApexClassId: `${this.flags.apexclass}`,
                    SourceId: sourceSandboxId
                },
                json: true
            });
        }
        else {
            if (!this.flags.licensetype) {
                throw new core_1.SfdxError("License type is required when clonefrom source org is not provided. you may need to provide -l | --licensetype");
            }
            result = await request({
                method: "post",
                uri,
                headers: {
                    Authorization: `Bearer ${conn.accessToken}`
                },
                body: {
                    AutoActivate: "true",
                    SandboxName: `${this.flags.name}`,
                    Description: `${this.flags.description}`,
                    LicenseType: `${this.flags.licensetype}`,
                    ApexClassId: `${this.flags.apexclass}`
                },
                json: true
            });
        }
        if (result.success) {
            sfpowerkit_1.SFPowerkit.log(`Successfully Enqueued Creation of Sandbox`, sfpowerkit_1.LoggerLevel.INFO);
            if (!this.flags.json)
                this.ux.logJson(result);
        }
        else {
            throw new core_1.SfdxError("Unable to Create sandbox");
        }
        return result;
    }
    async getSandboxId(conn, name) {
        const query_uri = `${conn.instanceUrl}/services/data/v${this.flags.apiversion}/tooling/query?q=SELECT+Id,SandboxName+FROM+SandboxInfo+WHERE+SandboxName+in+('${name}')`;
        const sandbox_query_result = await request({
            method: "get",
            url: query_uri,
            headers: {
                Authorization: `Bearer ${conn.accessToken}`
            },
            json: true
        });
        if (sandbox_query_result.records[0] == undefined)
            throw new core_1.SfdxError(`Unable to continue, Please check your sandbox name: ${name}`);
        sfpowerkit_1.SFPowerkit.log(`Fetched Sandbox Id for sandbox  ${name}  is ${sandbox_query_result.records[0].Id}`, sfpowerkit_1.LoggerLevel.INFO);
        return sandbox_query_result.records[0].Id;
    }
}
exports.default = Create;
Create.description = messages.getMessage("commandDescription");
Create.examples = [
    `$ sfdx sfpowerkit:org:sandbox:create -d Testsandbox -f sitSandbox -n test2 -v myOrg@example.com`,
    `$ sfdx sfpowerkit:org:sandbox:create -d Testsandbox -l DEVELOPER -n test2 -v myOrg@example.com`
];
Create.flagsConfig = {
    name: command_1.flags.string({
        required: true,
        char: "n",
        description: messages.getMessage("nameFlagDescription")
    }),
    description: command_1.flags.string({
        required: true,
        char: "d",
        description: messages.getMessage("descriptionFlagDescription")
    }),
    licensetype: command_1.flags.string({
        required: true,
        char: "l",
        options: ["DEVELOPER", "DEVELOPER_PRO", "PARTIAL", "FULL"],
        description: messages.getMessage("licenseFlagDescription")
    }),
    apexclass: command_1.flags.string({
        required: false,
        char: "a",
        default: "",
        description: messages.getMessage("apexClassFlagDescription")
    }),
    clonefrom: command_1.flags.string({
        required: false,
        char: "f",
        default: "",
        description: messages.getMessage("cloneFromFlagDescripton")
    })
};
// Comment this out if your command does not require a hub org username
Create.requiresDevhubUsername = true;
//# sourceMappingURL=create.js.map