"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const scratchOrgUtils_1 = __importDefault(require("../../../../utils/scratchOrgUtils"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "scratchorg_delete");
class Delete extends command_1.SfdxCommand {
    async run() {
        if (!this.flags.username && !this.flags.email) {
            throw new core_1.SfdxError("Required flags are missing, Please provide either username or email.");
        }
        await this.hubOrg.refreshAuth();
        const conn = this.hubOrg.getConnection();
        this.flags.apiversion =
            this.flags.apiversion || (await conn.retrieveMaxApiVersion());
        let info = await this.getActiveScratchOrgsForUser(conn, this.flags.email, this.flags.username);
        if (info.totalSize > 0) {
            this.ux.log(`Found ${info.totalSize} Scratch Org(s) for the given ${this.flags.username
                ? "Username: " + this.flags.username
                : "Email: " + this.flags.email} in devhub ${this.hubOrg.getUsername()}.\n`);
            this.ux.table(info.records, [
                "Id",
                "SignupUsername",
                "SignupEmail",
                "ExpirationDate",
            ]);
            let scratchOrgIds = info.records.map((elem) => elem.Id);
            await scratchOrgUtils_1.default.deleteScratchOrg(this.hubOrg, scratchOrgIds);
            this.ux.log("Scratch Org(s) deleted successfully.");
        }
        else {
            this.ux.log(`No Scratch Org(s) found for the given ${this.flags.username
                ? "Username: " + this.flags.username
                : "Email: " + this.flags.email} in devhub ${this.hubOrg.getUsername()}.`);
        }
        return 1;
    }
    async getActiveScratchOrgsForUser(conn, email, username) {
        let query = `SELECT Id, SignupUsername, SignupEmail, ExpirationDate FROM ActiveScratchOrg`;
        if (username) {
            query = `${query} WHERE SignupUsername = '${username}'`;
        }
        else {
            query = `${query} WHERE SignupEmail = '${email}'`;
        }
        const scratch_orgs = (await conn.query(query));
        return scratch_orgs;
    }
}
exports.default = Delete;
Delete.description = messages.getMessage("commandDescription");
Delete.examples = [
    `$ sfdx sfpowerkit:org:scratchorg:delete  -e xyz@kyz.com -v devhub`,
    `$ sfdx sfpowerkit:org:scratchorg:delete  -u xyz@kyz.com -v devhub`,
];
// Comment this out if your command does not require a hub org username
Delete.requiresDevhubUsername = true;
Delete.flagsConfig = {
    email: command_1.flags.string({
        required: false,
        char: "e",
        exclusive: ["username"],
        description: messages.getMessage("emailFlagDescription"),
    }),
    username: command_1.flags.string({
        required: false,
        char: "u",
        exclusive: ["email"],
        description: messages.getMessage("usernameFlagDescription"),
    }),
};
//# sourceMappingURL=delete.js.map