"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const fs = __importStar(require("fs-extra"));
const packageIdPrefix = "0Ho";
const packageVersionIdPrefix = "04t";
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "dependency_versionlist");
class List extends command_1.SfdxCommand {
    async run() {
        const conn = this.hubOrg.getConnection();
        let projectConfig = JSON.parse(fs.readFileSync("sfdx-project.json", "utf8"));
        let filterpaths = [];
        if (this.flags.filterpaths) {
            this.flags.filterpaths.forEach((path) => {
                filterpaths.push(path.split("\\").join("/"));
            });
        }
        for (let packageDirectory of projectConfig.packageDirectories) {
            if (filterpaths.length == 0 ||
                filterpaths.includes(packageDirectory.path)) {
                if (packageDirectory.dependencies &&
                    packageDirectory.dependencies[0] !== undefined) {
                    this.ux.log(`Package dependencies for the given package directory ${packageDirectory.path}`);
                    for (let dependency of packageDirectory.dependencies) {
                        if (projectConfig.packageAliases[dependency.package] !== "undefined") {
                            await this.getPackageVersionDetails(conn, dependency, projectConfig.packageAliases);
                            this.ux.log(`    ${dependency.versionId} : ${dependency.package}${dependency.versionNumber === undefined
                                ? ""
                                : " " + dependency.versionNumber}`);
                        }
                    }
                }
                else {
                    this.ux.log(`\nNo dependencies found for package directory ${packageDirectory.path}`);
                }
            }
        }
        if (this.flags.updateproject) {
            fs.writeFileSync("sfdx-project.json", JSON.stringify(projectConfig, (key, value) => {
                if (key == "versionId")
                    return undefined;
                else
                    return value;
            }, 2));
        }
        return projectConfig;
    }
    async getPackageVersionDetails(conn, dependency, packageAliases) {
        let packageId = packageAliases[dependency.package];
        if (packageId.startsWith(packageVersionIdPrefix)) {
            // Package2VersionId is set directly
            dependency["versionId"] = packageId;
        }
        else if (packageId.startsWith(packageIdPrefix)) {
            if (!dependency.versionNumber) {
                throw new command_1.core.SfdxError(`version number is mandatory for ${dependency.package}`);
            }
            // Get Package version id from package + versionNumber
            const vers = dependency.versionNumber.split(".");
            let query = "Select SubscriberPackageVersionId, IsPasswordProtected, IsReleased, MajorVersion, MinorVersion, PatchVersion,BuildNumber ";
            query += "from Package2Version ";
            query += `where Package2Id='${packageId}' and MajorVersion=${vers[0]} and MinorVersion=${vers[1]} and PatchVersion=${vers[2]} `;
            // If Build Number isn't set to LATEST, look for the exact Package Version
            if (vers[3] !== "LATEST") {
                query += `and BuildNumber=${vers[3]} `;
            }
            else if (this.flags.usedependencyvalidatedpackages) {
                query += `and ValidationSkipped = false `;
            }
            query += "ORDER BY BuildNumber DESC, createddate DESC Limit 1";
            // Query DevHub to get the expected Package2Version
            const resultPackageId = (await conn.tooling.query(query));
            if (resultPackageId.size === 0) {
                // Query returned no result
                const errorMessage = `Unable to find package ${dependency.package} of version ${dependency.versionNumber} in devhub ${this.hubOrg.getUsername()}. Are you sure it is created yet?`;
                throw new command_1.core.SfdxError(errorMessage);
            }
            else {
                let versionId = resultPackageId.records[0].SubscriberPackageVersionId;
                let versionNumber = `${resultPackageId.records[0].MajorVersion}.${resultPackageId.records[0].MinorVersion}.${resultPackageId.records[0].PatchVersion}.${resultPackageId.records[0].BuildNumber}`;
                dependency["versionId"] = versionId;
                dependency["versionNumber"] = versionNumber;
            }
        }
    }
}
exports.default = List;
List.description = messages.getMessage("commandDescription");
List.examples = [
    "$ sfdx sfpowerkit:package:dependencies:list -v MyDevHub -s src/dreamhouse",
    "$ sfdx sfpowerkit:package:dependencies:list -v MyDevHub --updateproject",
    "$ sfdx sfpowerkit:package:dependencies:list -v MyDevHub -s --usedependencyvalidatedpackages",
];
List.flagsConfig = {
    filterpaths: command_1.flags.array({
        char: "p",
        required: false,
        description: messages.getMessage("filterpathsDescription"),
    }),
    updateproject: command_1.flags.boolean({
        char: "w",
        required: false,
        description: messages.getMessage("updateprojectDescription"),
    }),
    usedependencyvalidatedpackages: command_1.flags.boolean({
        required: false,
        description: messages.getMessage("usedependencyvalidatedpackagesDescription"),
    }),
};
// Comment this out if your command does not require a hub org username
List.requiresDevhubUsername = true;
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
List.requiresProject = true;
//# sourceMappingURL=list.js.map