"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SFDXPackage = void 0;
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const xml2js = __importStar(require("xml2js"));
const util = __importStar(require("util"));
const fs = __importStar(require("fs-extra"));
const rimraf = __importStar(require("rimraf"));
const path = __importStar(require("path"));
const sfpowerkit_1 = require("../../../sfpowerkit");
const GetNodeWrapper_1 = require("../../../sfdxnode/GetNodeWrapper");
const parallel_1 = require("../../../sfdxnode/parallel");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "valid");
class Valid extends command_1.SfdxCommand {
    async run() {
        rimraf.sync("temp_sfpowerkit");
        sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
        GetNodeWrapper_1.loadSFDX();
        // Getting Project config
        const project = await core_1.SfdxProject.resolve();
        const projectJson = await project.retrieveSfdxProjectJson();
        let resourcePath = path.join(__dirname, "..", "..", "..", "..", "resources", "metadata.json");
        let fileData = fs.readFileSync(resourcePath, "utf8");
        this.coverageJSON = JSON.parse(fileData);
        if (this.isNotDefaultApiVersion()) {
            this.useCustomCoverageJSON();
        }
        let packageToBeScanned = this.flags.package;
        const packageDirectories = projectJson.get("packageDirectories") || [];
        const result_store = [];
        if (packageToBeScanned != undefined) {
            sfpowerkit_1.SFPowerkit.log(`Analyzing ${packageToBeScanned}`, sfpowerkit_1.LoggerLevel.INFO);
            for (const sf_package of packageDirectories) {
                if (packageToBeScanned != undefined &&
                    packageToBeScanned === sf_package["package"]) {
                    sfpowerkit_1.SFPowerkit.log(`located ${packageToBeScanned} in project ${sf_package["path"]}`, sfpowerkit_1.LoggerLevel.DEBUG);
                    let result;
                    try {
                        result = await this.validate(sf_package);
                    }
                    catch (e) {
                        sfpowerkit_1.SFPowerkit.log(`Error Occured Unable to analyze ${sf_package["package"]}`, sfpowerkit_1.LoggerLevel.ERROR);
                    }
                    break;
                }
            }
        }
        else {
            sfpowerkit_1.SFPowerkit.log("All packaging directories are  being analyzed", sfpowerkit_1.LoggerLevel.INFO);
            for (const sf_package of packageDirectories) {
                if (sf_package["package"] != undefined) {
                    sfpowerkit_1.SFPowerkit.log(`Analyzing ${sf_package["package"]}`, sfpowerkit_1.LoggerLevel.DEBUG);
                    let result;
                    try {
                        result = await this.validate(sf_package);
                        result_store.push(result);
                    }
                    catch (e) {
                        sfpowerkit_1.SFPowerkit.log(`Unable to analyze ${sf_package["package"]}, Skipping ${sf_package["package"]}. try running sfdx force:source:convert -r ${sf_package["path"]}`, sfpowerkit_1.LoggerLevel.ERROR);
                    }
                }
            }
        }
        if (!this.flags.json) {
            result_store.forEach(element => {
                if (element.valid == false)
                    throw new core_1.SfdxError("Analysis Failed, Unsupported metadata present");
            });
        }
        return { packages: result_store };
    }
    async validate(packageToBeScanned) {
        sfpowerkit_1.SFPowerkit.log(`Utilizing Version of the metadata coverage ${this.coverageJSON.versions.selected}`, sfpowerkit_1.LoggerLevel.DEBUG);
        sfpowerkit_1.SFPowerkit.log(`Converting package ${packageToBeScanned["package"]}`, sfpowerkit_1.LoggerLevel.INFO);
        var sfdx_package = new SFDXPackage();
        sfdx_package.packageName = packageToBeScanned["package"];
        await parallel_1.sfdx.force.source.convert({
            quiet: true,
            outputdir: "temp_sfpowerkit/mdapi",
            packagename: packageToBeScanned["package"],
            rootdir: packageToBeScanned["path"]
        });
        //Bypass package validation
        if (this.flags.bypass) {
            sfdx_package.typesToBypass = this.flags.bypass;
        }
        let targetFilename = "temp_sfpowerkit/mdapi/package.xml";
        if (fs.existsSync(targetFilename)) {
            const parser = new xml2js.Parser({ explicitArray: false });
            const parseString = util.promisify(parser.parseString);
            const existing = await parseString(fs.readFileSync(targetFilename));
            if (Array.isArray(existing.Package.types)) {
                for (const types of existing.Package.types) {
                    if (this.coverageJSON.types[types["name"]] != undefined)
                        if (this.coverageJSON.types[types["name"]].channels
                            .unlockedPackagingWithoutNamespace)
                            sfdx_package.supportedTypes.push(`${types["name"]}`);
                        else
                            sfdx_package.unsupportedtypes.push(`${types["name"]}`);
                }
            }
            else {
                if (this.coverageJSON.types[existing.Package.types["name"]] != undefined)
                    if (this.coverageJSON.types[existing.Package.types["name"]].channels
                        .unlockedPackagingWithoutNamespace)
                        sfdx_package.supportedTypes.push(`${existing.Package.types["name"]}`);
                    else
                        sfdx_package.unsupportedtypes.push(`${existing.Package.types["name"]}`);
            }
            sfdx_package.processed = true;
            if (sfdx_package.supportedTypes.length > 0) {
                this.ux.log(`Supported metadata in package ${packageToBeScanned["package"]}`);
                sfdx_package.supportedTypes.forEach(element => {
                    this.ux.log(element);
                });
                sfdx_package.valid = true;
                this.ux.log(`--------------------------------------------------------------------------------`);
            }
            //Bypass metadata in package validator
            if (sfdx_package.typesToBypass.length > 0 &&
                sfdx_package.unsupportedtypes.length > 0) {
                let itemsToRemove = [];
                sfdx_package.typesToBypass = sfdx_package.typesToBypass.map(element => element.toLowerCase());
                sfdx_package.unsupportedtypes = sfdx_package.unsupportedtypes.map(element => element.toLowerCase());
                itemsToRemove = sfdx_package.typesToBypass.filter(element => sfdx_package.unsupportedtypes.includes(element));
                if (itemsToRemove.length > 0) {
                    this.ux.log(`Unsupported metadata in package ${packageToBeScanned["package"]}  to bypass`);
                    itemsToRemove.forEach(element => {
                        this.ux.log(element);
                    });
                    sfdx_package.unsupportedtypes = sfdx_package.unsupportedtypes.filter(element => !itemsToRemove.includes(element));
                    this.ux.log(`--------------------------------------------------------------------------------`);
                }
            }
            if (sfdx_package.unsupportedtypes.length > 0) {
                this.ux.log(`Unsupported metadata in package ${packageToBeScanned["package"]}`);
                sfdx_package.unsupportedtypes.forEach(element => {
                    this.ux.log(element);
                });
                sfdx_package.valid = false;
                this.ux.log(`--------------------------------------------------------------------------------`);
            }
        }
        rimraf.sync("temp_sfpowerkit");
        return sfdx_package;
    }
    useCustomCoverageJSON() {
        try {
            let resourcePath = path.join(__dirname, "..", "..", "..", "..", "resources", `metadata_v${this.flags.apiversion}.json`);
            let fileData = fs.readFileSync(resourcePath, "utf8");
            this.coverageJSON = JSON.parse(fileData);
        }
        catch (fileError) {
            throw new core_1.SfdxError(`Unable to read version ${this.flags.apiversion} of metadata coverage JSON`);
        }
    }
    isNotDefaultApiVersion() {
        return (this.flags.apiversion &&
            this.coverageJSON.versions.selected != this.flags.apiversion);
    }
}
exports.default = Valid;
Valid.description = messages.getMessage("commandDescription");
Valid.examples = [
    `$ sfdx sfpowerkit:package:valid -n testPackage
  Now analyzing testPackage
Converting package testPackage
Elements supported included in your package testPackage
[
  "AuraDefinitionBundle",
  "CustomApplication",
  "ApexClass",
  "ContentAsset",
  "WorkflowRule"
]
  `
];
Valid.flagsConfig = {
    package: command_1.flags.string({
        required: false,
        char: "n",
        description: messages.getMessage("packageFlagDescription")
    }),
    bypass: command_1.flags.array({
        required: false,
        char: "b",
        description: messages.getMessage("itemsToBypassValidationDescription")
    }),
    apiversion: command_1.flags.builtin({
        description: messages.getMessage("apiversion")
    }),
    loglevel: command_1.flags.enum({
        description: "loglevel to execute the command",
        default: "info",
        required: false,
        options: [
            "trace",
            "debug",
            "info",
            "warn",
            "error",
            "fatal",
            "TRACE",
            "DEBUG",
            "INFO",
            "WARN",
            "ERROR",
            "FATAL"
        ]
    })
};
// Comment this out if your command does not require an org username
//protected static requiresUsername = true;
// Comment this out if your command does not support a hub org username
// protected static supportsDevhubUsername = true;
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
Valid.requiresProject = true;
class SFDXPackage {
    constructor() {
        this.unsupportedtypes = [];
        this.supportedTypes = [];
        this.typesToBypass = [];
    }
}
exports.SFDXPackage = SFDXPackage;
//# sourceMappingURL=valid.js.map