"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const sfpowerkit_1 = require("../../../sfpowerkit");
const poolFetchImpl_1 = __importDefault(require("../../../impl/pool/scratchorg/poolFetchImpl"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "scratchorg_poolFetch");
class Fetch extends command_1.SfdxCommand {
    async run() {
        sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
        await this.hubOrg.refreshAuth();
        const hubConn = this.hubOrg.getConnection();
        this.flags.apiversion =
            this.flags.apiversion || (await hubConn.retrieveMaxApiVersion());
        let fetchImpl = new poolFetchImpl_1.default(this.hubOrg, this.flags.tag, this.flags.mypool, this.flags.sendtouser);
        let result = await fetchImpl.execute();
        if (!this.flags.json && !this.flags.sendtouser) {
            this.ux.log(`======== Scratch org details ========`);
            let list = [];
            for (let [key, value] of Object.entries(result)) {
                if (value) {
                    list.push({ key: key, value: value });
                }
            }
            this.ux.table(list, ["key", "value"]);
        }
        if (!this.flags.sendtouser)
            return result;
        else
            return true;
    }
}
exports.default = Fetch;
Fetch.description = messages.getMessage("commandDescription");
Fetch.requiresDevhubUsername = true;
Fetch.examples = [
    `$ sfdx sfpowerkit:pool:fetch -t core `,
    `$ sfdx sfpowerkit:pool:fetch -t core -v devhub`,
    `$ sfdx sfpowerkit:pool:fetch -t core -v devhub -m`,
    `$ sfdx sfpowerkit:pool:fetch -t core -v devhub -s testuser@test.com`,
];
Fetch.flagsConfig = {
    tag: command_1.flags.string({
        char: "t",
        description: messages.getMessage("tagDescription"),
        required: true,
    }),
    mypool: command_1.flags.boolean({
        char: "m",
        description: messages.getMessage("mypoolDescription"),
        required: false,
    }),
    sendtouser: command_1.flags.string({
        char: "s",
        description: messages.getMessage("sendToUserDescription"),
        required: false,
    }),
    loglevel: command_1.flags.enum({
        description: "logging level for this command invocation",
        default: "info",
        required: false,
        options: [
            "trace",
            "debug",
            "info",
            "warn",
            "error",
            "fatal",
            "TRACE",
            "DEBUG",
            "INFO",
            "WARN",
            "ERROR",
            "FATAL",
        ],
    }),
};
//# sourceMappingURL=fetch.js.map