"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const sfpowerkit_1 = require("../../../sfpowerkit");
const poolListImpl_1 = __importDefault(require("../../../impl/pool/scratchorg/poolListImpl"));
const util_1 = require("util");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "scratchorg_poollist");
class List extends command_1.SfdxCommand {
    async run() {
        sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
        await this.hubOrg.refreshAuth();
        const hubConn = this.hubOrg.getConnection();
        this.flags.apiversion =
            this.flags.apiversion || (await hubConn.retrieveMaxApiVersion());
        let listImpl = new poolListImpl_1.default(this.hubOrg, this.flags.apiversion, this.flags.tag, this.flags.mypool, this.flags.allscratchorgs);
        let result = await listImpl.execute();
        if (!this.flags.mypool && result.length > 0) {
            result.forEach((element) => {
                delete element.password;
            });
        }
        let scratchOrgInuse = result.filter((element) => element.status === "In use");
        let scratchOrgNotInuse = result.filter((element) => element.status === "Available");
        let scratchOrgInProvision = result.filter((element) => element.status === "Provisioning in progress");
        if (!this.flags.json) {
            if (result.length > 0) {
                this.ux.log(`======== Scratch org Details ========`);
                if (util_1.isNullOrUndefined(this.flags.tag)) {
                    this.ux.log(`List of all the pools in the org`);
                    this.logTagCount(result);
                    this.ux.log("===================================");
                }
                if (this.flags.allscratchorgs) {
                    this.ux.log(`Used Scratch Orgs in the pool: ${scratchOrgInuse.length}`);
                }
                this.ux.log(`Unused Scratch Orgs in the Pool : ${scratchOrgNotInuse.length} \n`);
                if (scratchOrgInProvision.length && scratchOrgInProvision.length > 0) {
                    this.ux.log(`Scratch Orgs being provisioned in the Pool : ${scratchOrgInProvision.length} \n`);
                }
                if (this.flags.mypool) {
                    this.ux.table(result, [
                        "tag",
                        "orgId",
                        "username",
                        "password",
                        "expityDate",
                        "status",
                        "loginURL",
                    ]);
                }
                else {
                    this.ux.table(result, [
                        "tag",
                        "orgId",
                        "username",
                        "expityDate",
                        "status",
                        "loginURL",
                    ]);
                }
            }
            else {
                sfpowerkit_1.SFPowerkit.log(`${this.flags.tag} pool has No Scratch orgs available, time to create your pool.`, sfpowerkit_1.LoggerLevel.INFO);
            }
        }
        let output = {
            total: scratchOrgInuse.length +
                scratchOrgNotInuse.length +
                scratchOrgInProvision.length,
            inuse: scratchOrgInuse.length,
            unused: scratchOrgNotInuse.length,
            inprovision: scratchOrgInProvision.length,
            scratchOrgDetails: result,
        };
        return output;
    }
    logTagCount(result) {
        let tagCounts = result.reduce(function (obj, v) {
            obj[v.tag] = (obj[v.tag] || 0) + 1;
            return obj;
        }, {});
        let tagArray = new Array();
        Object.keys(tagCounts).forEach(function (key) {
            tagArray.push({
                tag: key,
                count: tagCounts[key],
            });
        });
        this.ux.table(tagArray, ["tag", "count"]);
    }
}
exports.default = List;
List.description = messages.getMessage("commandDescription");
List.requiresDevhubUsername = true;
List.examples = [
    `$ sfdx sfpowerkit:pool:list -t core `,
    `$ sfdx sfpowerkit:pool:list -t core -v devhub`,
    `$ sfdx sfpowerkit:pool:list -t core -v devhub -m`,
    `$ sfdx sfpowerkit:pool:list -t core -v devhub -m -a`,
];
List.flagsConfig = {
    tag: command_1.flags.string({
        char: "t",
        description: messages.getMessage("tagDescription"),
        required: false,
    }),
    mypool: command_1.flags.boolean({
        char: "m",
        description: messages.getMessage("mypoolDescription"),
        required: false,
    }),
    allscratchorgs: command_1.flags.boolean({
        char: "a",
        description: messages.getMessage("allscratchorgsDescription"),
        required: false,
    }),
    loglevel: command_1.flags.enum({
        description: "logging level for this command invocation",
        default: "info",
        required: false,
        options: [
            "trace",
            "debug",
            "info",
            "warn",
            "error",
            "fatal",
            "TRACE",
            "DEBUG",
            "INFO",
            "WARN",
            "ERROR",
            "FATAL",
        ],
    }),
};
//# sourceMappingURL=list.js.map