"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const sfpowerkit_1 = require("../../../../sfpowerkit");
const fs = __importStar(require("fs-extra"));
const simple_git_1 = __importDefault(require("simple-git"));
const util_1 = require("util");
const DataModelSourceDiffImpl_1 = __importDefault(require("../../../../impl/project/metadata/DataModelSourceDiffImpl"));
const path = __importStar(require("path"));
const fileutils_1 = __importDefault(require("../../../../utils/fileutils"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "project_datamodel_diff");
class Diff extends command_1.SfdxCommand {
    async run() {
        sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
        let isOutputCSV = this.flags.csv;
        let outputDirectory = this.flags.outputdir
            ? this.flags.outputdir
            : process.cwd();
        let git = simple_git_1.default();
        const revisionFrom = await git.revparse([
            "--short",
            this.flags.revisionfrom
        ]);
        const revisionTo = await git.revparse([
            "--short",
            this.flags.revisionto
        ]);
        let packageDirectories;
        if (!util_1.isNullOrUndefined(this.flags.packagedirectories)) {
            packageDirectories = this.flags.packagedirectories.split(",");
            packageDirectories = packageDirectories.map(dir => {
                return dir.trim().toLocaleLowerCase();
            });
            let projectConfig = JSON.parse(fs.readFileSync("sfdx-project.json", "utf8"));
            packageDirectories.forEach(dir => {
                let isValidPackageDir;
                projectConfig["packageDirectories"].forEach(configPackageDir => {
                    if (dir == configPackageDir["path"].toLocaleLowerCase())
                        isValidPackageDir = true;
                });
                if (!isValidPackageDir)
                    throw new Error("Invalid package directory supplied");
            });
        }
        let dataModelSourceDiffImpl = new DataModelSourceDiffImpl_1.default(git, revisionFrom, revisionTo, packageDirectories);
        let sourceDiffResult = await dataModelSourceDiffImpl.exec();
        if (sourceDiffResult.length < 1) {
            sfpowerkit_1.SFPowerkit.log(`No Datamodel change found between ${revisionFrom} and ${revisionTo}`, sfpowerkit_1.LoggerLevel.WARN);
            return sourceDiffResult;
        }
        sfpowerkit_1.SFPowerkit.log(`Found ${sourceDiffResult.length} Datamodel change between ${revisionFrom} and ${revisionTo} \n`, sfpowerkit_1.LoggerLevel.INFO);
        let csvPath = `${outputDirectory}/datamodel-diff-output.json`;
        let dir = path.parse(csvPath).dir;
        if (!fs.existsSync(dir)) {
            fileutils_1.default.mkDirByPathSync(dir);
        }
        fs.writeFileSync(csvPath, JSON.stringify(sourceDiffResult, null, 4));
        let rowsToDisplay = [];
        for (let file of sourceDiffResult) {
            for (let change of file["diff"]) {
                rowsToDisplay.push({
                    object: file["object"],
                    api_name: file["api_name"],
                    type: file["type"],
                    operation: change["operation"],
                    coordinates: change["coordinates"],
                    from: change["before"],
                    to: change["after"],
                    filepath: file["filepath"]
                });
            }
        }
        if (isOutputCSV) {
            let csvOutput = `Object,API_Name,Type,Operation,Coordinates,Commit ID (${revisionFrom}),Commit ID (${revisionTo}),Filepath\n`;
            for (let row of rowsToDisplay) {
                let rowCells = Object.values(row);
                csvOutput = csvOutput + `${rowCells.toString()}\n`;
            }
            fs.writeFileSync(`${outputDirectory}/datamodel-diff-output.csv`, csvOutput);
        }
        this.ux.table(rowsToDisplay.slice(0, 50), [
            "object",
            "api_name",
            "type",
            "operation",
            "coordinates",
            "from",
            "to"
        ]);
        this.ux.log("\n");
        if (rowsToDisplay.length > 50) {
            sfpowerkit_1.SFPowerkit.log("Displaying output limited to 50 rows", sfpowerkit_1.LoggerLevel.WARN);
        }
        sfpowerkit_1.SFPowerkit.log(`JSON output written to ${outputDirectory}/datamodel-diff-output.json`, sfpowerkit_1.LoggerLevel.INFO);
        if (isOutputCSV) {
            sfpowerkit_1.SFPowerkit.log(`CSV output written to ${outputDirectory}/datamodel-diff-output.csv`, sfpowerkit_1.LoggerLevel.INFO);
        }
        return sourceDiffResult;
    }
}
exports.default = Diff;
Diff.description = messages.getMessage("commandDescription");
Diff.examples = [
    `$ sfdx sfpowerkit:project:datamodel:diff --revisionfrom revisionfrom --revisionto revisionto --csv`
];
Diff.flagsConfig = {
    revisionfrom: command_1.flags.string({
        char: "r",
        description: messages.getMessage("revisionFromDescription"),
        required: true
    }),
    revisionto: command_1.flags.string({
        char: "t",
        description: messages.getMessage("revisionToDescription"),
        required: false,
        default: "HEAD"
    }),
    packagedirectories: command_1.flags.string({
        required: false,
        char: "p",
        description: messages.getMessage("packageDirectoriesDescription")
    }),
    outputdir: command_1.flags.directory({
        required: false,
        char: "d",
        description: messages.getMessage("outputDirDescription")
    }),
    csv: command_1.flags.boolean({
        required: false,
        description: messages.getMessage("csvDescription"),
        default: false
    }),
    loglevel: command_1.flags.enum({
        description: "logging level for this command invocation",
        default: "info",
        required: false,
        options: [
            "trace",
            "debug",
            "info",
            "warn",
            "error",
            "fatal",
            "TRACE",
            "DEBUG",
            "INFO",
            "WARN",
            "ERROR",
            "FATAL"
        ]
    })
};
Diff.requiresUsername = false;
Diff.requiresProject = true;
//# sourceMappingURL=diff.js.map