"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const command_1 = require("@salesforce/command");
const sfpowerkit_1 = require("../../../../sfpowerkit");
const core_1 = require("@salesforce/core");
const ApexTypeFetcher_1 = __importDefault(require("../../../../impl/parser/ApexTypeFetcher"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "source_apextest_list");
class List extends command_1.SfdxCommand {
    async run() {
        sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
        //set apex class directory
        if (!fs_1.existsSync(this.flags.path)) {
            throw new core_1.SfdxError(`path ${this.flags.path} does not exist. you must provide a valid path.`);
        }
        let apexTypeFetcher = new ApexTypeFetcher_1.default();
        let apexSortedByType = apexTypeFetcher.getApexTypeOfClsFiles(this.flags.path);
        let testClasses = apexSortedByType["testClass"];
        let testClassesList = testClasses.map((cls) => cls.name);
        if (testClasses.length > 0) {
            sfpowerkit_1.SFPowerkit.log(`Found ${testClasses.length} apex test classes in ${this.flags.path}`, sfpowerkit_1.LoggerLevel.INFO);
            if (this.flags.resultasstring) {
                this.ux.log(testClassesList.join(","));
            }
            else {
                this.ux.table(testClasses, ["name", "filepath"]);
            }
        }
        else {
            sfpowerkit_1.SFPowerkit.log(`No apex test classes found in ${this.flags.path}`, sfpowerkit_1.LoggerLevel.INFO);
        }
        return this.flags.resultasstring
            ? testClassesList.join(",")
            : testClassesList;
    }
}
exports.default = List;
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
List.requiresProject = true;
List.description = messages.getMessage("commandDescription");
List.examples = [
    `$ sfdx sfpowerkit:source:apextest:list -p force-app`,
];
List.flagsConfig = {
    path: command_1.flags.string({
        required: true,
        char: "p",
        description: messages.getMessage("pathFlagDescription"),
    }),
    resultasstring: command_1.flags.boolean({
        description: messages.getMessage("resultasstringDescription"),
        required: false,
    }),
    loglevel: command_1.flags.enum({
        description: messages.getMessage("loglevel"),
        default: "info",
        required: false,
        options: [
            "trace",
            "debug",
            "info",
            "warn",
            "error",
            "fatal",
            "TRACE",
            "DEBUG",
            "INFO",
            "WARN",
            "ERROR",
            "FATAL",
        ],
    }),
};
//# sourceMappingURL=list.js.map