"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sfpowerkit_1 = require("../../../sfpowerkit");
const core_1 = require("@salesforce/core");
const packageUtils_1 = require("../../../utils/packageUtils");
let retry = require("async-retry");
class PackageInfo {
    constructor(conn, apiversion, jsonOutput) {
        this.conn = conn;
        this.apiversion = apiversion;
    }
    async getPackages() {
        //await this.getInstalledPackageInfo();
        let packageDetails = await packageUtils_1.getInstalledPackages(this.conn, true);
        sfpowerkit_1.SFPowerkit.log("PackageDetails:" + JSON.stringify(packageDetails), core_1.LoggerLevel.TRACE);
        return packageDetails;
    }
    async getPackagesDetailsfromDevHub(hubconn, pkgDetails) {
        let pkgIds = [];
        pkgDetails.forEach(pkg => {
            if (pkg.type === "Unlocked") {
                pkgIds.push(pkg.packageVersionId);
            }
        });
        let pkdIdsAsString = pkgIds.join(`','`);
        if (pkgIds.length > 0) {
            let installedPackagesQuery = `SELECT SubscriberPackageVersionId, HasPassedCodeCoverageCheck,CodeCoverage,ValidationSkipped FROM Package2Version WHERE SubscriberPackageVersionId IN('${pkdIdsAsString}')`;
            let response = await retry(async (bail) => {
                return await hubconn.tooling.query(installedPackagesQuery);
            }, { retries: 3, minTimeout: 3000 });
            if (response.records && response.records.length > 0) {
                response.records.forEach(record => {
                    for (let pkg of pkgDetails) {
                        if (pkg.packageVersionId === record.SubscriberPackageVersionId) {
                            pkg.codeCoverageCheckPassed = record.HasPassedCodeCoverageCheck;
                            pkg.CodeCoverage = record.CodeCoverage
                                ? record.CodeCoverage.apexCodeCoveragePercentage
                                : 0;
                            pkg.validationSkipped = record.ValidationSkipped;
                        }
                    }
                });
            }
        }
        return pkgDetails;
    }
}
exports.default = PackageInfo;
//# sourceMappingURL=packageInfo.js.map