"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const scratchOrgUtils_1 = __importDefault(require("../../../utils/scratchOrgUtils"));
const core_1 = require("@salesforce/core");
const sfpowerkit_1 = require("../../../sfpowerkit");
const fs = __importStar(require("fs-extra"));
const child_process_1 = require("child_process");
const bottleneck_1 = __importDefault(require("bottleneck"));
const util_1 = require("util");
const relaxIPRangeImpl_1 = __importDefault(require("../../org/relaxIPRangeImpl"));
const fileutils_1 = __importDefault(require("../../../utils/fileutils"));
const path = __importStar(require("path"));
const rimraf = __importStar(require("rimraf"));
class PoolCreateImpl {
    constructor(poolconfigFilePath, hubOrg, apiversion, sfdx, batchSize) {
        this.poolconfigFilePath = poolconfigFilePath;
        this.hubOrg = hubOrg;
        this.apiversion = apiversion;
        this.sfdx = sfdx;
        this.batchSize = batchSize;
        this.totalAllocated = 0;
        this.arrayToObject = (array, keyfield) => array.reduce((obj, item) => {
            obj[item[keyfield]] = item;
            return obj;
        }, {});
        this.limiter = new bottleneck_1.default({
            maxConcurrent: batchSize,
        });
        this.scriptExecutorWrappedForBottleneck = this.limiter.wrap(this.scriptExecutor);
        this.ipRangeRelaxerWrappedForBottleneck = this.limiter.wrap(this.ipRangeRelaxer);
    }
    async poolScratchOrgs() {
        await scratchOrgUtils_1.default.checkForNewVersionCompatible(this.hubOrg);
        let scriptExecPromises = new Array();
        let ipRangeExecPromises = new Array();
        await this.hubOrg.refreshAuth();
        this.hubConn = this.hubOrg.getConnection();
        let preRequisiteCheck = await scratchOrgUtils_1.default.checkForPreRequisite(this.hubOrg);
        if (!preRequisiteCheck) {
            sfpowerkit_1.SFPowerkit.log("Required Prerequisite fields are missing in the DevHub, Please look into the wiki to getting the fields deployed in DevHub", core_1.LoggerLevel.ERROR);
            return false;
        }
        //Read pool config file
        if (!fs.existsSync(this.poolconfigFilePath)) {
            sfpowerkit_1.SFPowerkit.log("Poll Config Path not provided, Unable to create pool without this file", core_1.LoggerLevel.ERROR);
            return false;
        }
        this.poolConfig = JSON.parse(fs.readFileSync(this.poolconfigFilePath).toString());
        //Validate Inputs
        if (util_1.isNullOrUndefined(this.poolConfig.pool.config_file_path)) {
            sfpowerkit_1.SFPowerkit.log("Scratch Org Config Path not provided, Unable to create pool without this file", core_1.LoggerLevel.ERROR);
            return true;
        }
        if (util_1.isNullOrUndefined(this.poolConfig.pool.expiry) ||
            util_1.isNullOrUndefined(this.poolConfig.pool.tag)) {
            sfpowerkit_1.SFPowerkit.log("Some Key parameters are missing in the schema,Please consult the documentation", core_1.LoggerLevel.ERROR);
            return true;
        }
        this.validateScriptFile();
        if (this.poolConfig.poolUsers && this.poolConfig.poolUsers.length > 0)
            this.poolConfig.pool.user_mode = true;
        else
            this.poolConfig.pool.user_mode = false;
        //Set Tag Only mode activated for the default use case
        if (this.poolConfig.pool.user_mode == false)
            this.setASingleUserForTagOnlyMode();
        sfpowerkit_1.SFPowerkit.log("Pool Config:" + JSON.stringify(this.poolConfig), core_1.LoggerLevel.TRACE);
        if (!this.poolConfig.pool.relax_all_ip_ranges &&
            util_1.isNullOrUndefined(this.poolConfig.pool.relax_ip_ranges) &&
            !this.poolConfig.pool.user_mode) {
            sfpowerkit_1.SFPowerkit.log("IP Ranges are not relaxed, The created scratch org's will have the pool creators email as Admin Email and has to be verifed before use", core_1.LoggerLevel.WARN);
        }
        //fetch current status limits
        await this.fetchCurrentLimits();
        //Compute allocation
        this.totalToBeAllocated = await this.computeAllocation();
        if (this.totalToBeAllocated === 0) {
            if (this.limits.ActiveScratchOrgs.Remaining > 0)
                sfpowerkit_1.SFPowerkit.log(`The tag provided ${this.poolConfig.pool.tag} is currently at the maximum capacity , No scratch orgs will be allocated`, core_1.LoggerLevel.INFO);
            else
                sfpowerkit_1.SFPowerkit.log(`There is no capacity to create a pool at this time, Please try again later`, core_1.LoggerLevel.INFO);
            return;
        }
        //Generate Scratch Orgs
        await this.generateScratchOrgs();
        // Setup Logging Directory
        rimraf.sync("script_exec_outputs");
        fileutils_1.default.mkDirByPathSync("script_exec_outputs");
        // Assign workers to executed scripts
        let ts = Math.floor(Date.now() / 1000);
        for (let poolUser of this.poolConfig.poolUsers) {
            for (let scratchOrg of poolUser.scratchOrgs) {
                sfpowerkit_1.SFPowerkit.log(JSON.stringify(scratchOrg), core_1.LoggerLevel.DEBUG);
                if (this.poolConfig.pool.relax_all_ip_ranges ||
                    this.poolConfig.pool.relax_ip_ranges) {
                    let resultForIPRelaxation = this.ipRangeRelaxerWrappedForBottleneck(scratchOrg);
                    ipRangeExecPromises.push(resultForIPRelaxation);
                }
                //Wait for scripts to finish execution
                if (this.poolConfig.pool.relax_all_ip_ranges ||
                    this.poolConfig.pool.relax_ip_ranges)
                    this.ipRangeExecResults = await Promise.all(ipRangeExecPromises);
                if (this.scriptFileExists) {
                    let result = this.scriptExecutorWrappedForBottleneck(this.poolConfig.pool.script_file_path, scratchOrg, this.hubOrg.getUsername());
                    scriptExecPromises.push(result);
                }
                else {
                    //Just commit it to the pool as there is no script, and ensuring it doesnt get deleted
                    scratchOrg.isScriptExecuted = true;
                    await scratchOrgUtils_1.default.setScratchOrgInfo({
                        Id: scratchOrg.recordId,
                        Pooltag__c: this.poolConfig.pool.tag,
                        Allocation_status__c: scratchOrgUtils_1.default.isNewVersionCompatible
                            ? "Available"
                            : "",
                        Password__c: scratchOrg.password,
                    }, this.hubOrg);
                }
            }
        }
        //Get IP Range results
        if (this.poolConfig.pool.relax_ip_ranges ||
            !util_1.isNullOrUndefined(this.poolConfig.pool.relax_ip_ranges))
            this.ipRangeExecResultsAsObject = this.arrayToObject(this.ipRangeExecResults, "username");
        let scriptExecResults = await Promise.all(scriptExecPromises);
        if (this.scriptFileExists) {
            sfpowerkit_1.SFPowerkit.log(JSON.stringify(scriptExecResults), core_1.LoggerLevel.TRACE);
            ts = Math.floor(Date.now() / 1000) - ts;
            sfpowerkit_1.SFPowerkit.log(`Pool Execution completed in ${ts} Seconds`, core_1.LoggerLevel.INFO);
        }
        //Commit Succesfull Scratch Orgs
        let commit_result = await this.finalizeGeneratedScratchOrgs();
        if (this.totalAllocated > 0) {
            sfpowerkit_1.SFPowerkit.log(`Request for provisioning ${this.totalToBeAllocated} scratchOrgs of which ${this.totalAllocated} were allocated with ${commit_result.success} success and ${commit_result.failed} failures`, core_1.LoggerLevel.INFO);
        }
        else {
            sfpowerkit_1.SFPowerkit.log(`Request for provisioning ${this.totalToBeAllocated} scratchOrgs not successfull.`, core_1.LoggerLevel.ERROR);
        }
        return true;
    }
    validateScriptFile() {
        if (util_1.isNullOrUndefined(this.poolConfig.pool.script_file_path)) {
            sfpowerkit_1.SFPowerkit.log("Script Path not provided, will create a pool of scratch orgs without any post creation steps", core_1.LoggerLevel.WARN);
            this.scriptFileExists = false;
        }
        else if (fs.existsSync(this.poolConfig.pool.script_file_path)) {
            this.scriptFileExists = true;
        }
        else {
            sfpowerkit_1.SFPowerkit.log("Unable to locate Script File path, will crete a pool of scratch orgs without any post creation steps", core_1.LoggerLevel.WARN);
            this.scriptFileExists = false;
        }
    }
    setASingleUserForTagOnlyMode() {
        //Remove any existing pool Config for pool users
        if (this.poolConfig.poolUsers)
            delete this.poolConfig.poolUsers;
        let poolUser = {
            min_allocation: this.poolConfig.pool.max_allocation,
            max_allocation: this.poolConfig.pool.max_allocation,
            is_build_pooluser: false,
            expiry: this.poolConfig.pool.expiry,
            priority: 1,
        };
        //Add a single user
        this.poolConfig.poolUsers = [];
        this.poolConfig.poolUsers.push(poolUser);
        this.poolConfig.pool.user_mode = false;
    }
    async fetchCurrentLimits() {
        try {
            this.limits = await scratchOrgUtils_1.default.getScratchOrgLimits(this.hubOrg, this.apiversion);
        }
        catch (error) {
            sfpowerkit_1.SFPowerkit.log("Unable to connect to DevHub", core_1.LoggerLevel.ERROR);
            return;
        }
        sfpowerkit_1.SFPowerkit.log(`Active Scratch Orgs Remaining: ${this.limits.ActiveScratchOrgs.Remaining} out of ${this.limits.ActiveScratchOrgs.Max}`, core_1.LoggerLevel.TRACE);
    }
    async computeAllocation() {
        //Compute current pool requirement
        if (this.poolConfig.pool.user_mode) {
            //Retrieve Number of active SOs in the org
            let scratchOrgsResult = await scratchOrgUtils_1.default.getScratchOrgsByTag(this.poolConfig.pool.tag, this.hubOrg, false, false);
            scratchOrgsResult.records = scratchOrgsResult.records.sort();
            let scratchOrgsRecordAsMapByUser = scratchOrgsResult.records.reduce(function (obj, v) {
                obj[v.SignupEmail] = (obj[v.SignupEmail] || 0) + 1;
                return obj;
            }, {});
            sfpowerkit_1.SFPowerkit.log(JSON.stringify(scratchOrgsRecordAsMapByUser), core_1.LoggerLevel.TRACE);
            return this.allocateScratchOrgsPerUser(this.limits.ActiveScratchOrgs.Remaining, scratchOrgsRecordAsMapByUser, this.poolConfig.poolUsers);
        }
        else {
            let activeCount = await scratchOrgUtils_1.default.getCountOfActiveScratchOrgsByTag(this.poolConfig.pool.tag, this.hubOrg);
            return this.allocateScratchOrgsPerTag(this.limits.ActiveScratchOrgs.Remaining, activeCount, this.poolConfig.pool.tag, this.poolConfig.poolUsers[0]);
        }
    }
    async generateScratchOrgs() {
        //Generate Scratch Orgs
        for (let poolUser of this.poolConfig.poolUsers) {
            let count = 1;
            poolUser.scratchOrgs = new Array();
            for (let i = 0; i < poolUser.to_allocate; i++) {
                sfpowerkit_1.SFPowerkit.log(`Creating Scratch  Org  ${count} of ${this.totalToBeAllocated}..`, core_1.LoggerLevel.INFO);
                try {
                    let scratchOrg = await scratchOrgUtils_1.default.createScratchOrg(this.sfdx, count, poolUser.username, this.poolConfig.pool.config_file_path, poolUser.expiry ? poolUser.expiry : this.poolConfig.pool.expiry, this.hubOrg);
                    poolUser.scratchOrgs.push(scratchOrg);
                    this.totalAllocated++;
                }
                catch (error) {
                    sfpowerkit_1.SFPowerkit.log(`Unable to provision scratch org  ${count} ..   `, core_1.LoggerLevel.INFO);
                }
                count++;
            }
            await scratchOrgUtils_1.default.getScratchOrgRecordId(poolUser.scratchOrgs, this.hubOrg);
            if (scratchOrgUtils_1.default.isNewVersionCompatible) {
                let scratchOrgInprogress = [];
                poolUser.scratchOrgs.forEach((scratchOrg) => {
                    scratchOrgInprogress.push({
                        Id: scratchOrg.recordId,
                        Pooltag__c: this.poolConfig.pool.tag,
                        Allocation_status__c: "In Progress",
                    });
                });
                if (scratchOrgInprogress.length > 0) {
                    //set pool tag
                    await scratchOrgUtils_1.default.setScratchOrgInfo(scratchOrgInprogress, this.hubOrg);
                }
            }
        }
    }
    async finalizeGeneratedScratchOrgs() {
        //Store Username Passwords
        let failed = 0;
        let success = 0;
        for (let poolUser of this.poolConfig.poolUsers) {
            for (let scratchOrg of poolUser.scratchOrgs) {
                if (scratchOrg.isScriptExecuted) {
                    success++;
                    continue;
                }
                sfpowerkit_1.SFPowerkit.log(`Failed to execute scripts for ${scratchOrg.username} with alias ${scratchOrg.alias}.. Returning to Pool`, core_1.LoggerLevel.ERROR);
                try {
                    //Delete scratchorgs that failed to execute script
                    let activeScratchOrgRecordId = await scratchOrgUtils_1.default.getActiveScratchOrgRecordIdGivenScratchOrg(this.hubOrg, this.apiversion, scratchOrg.orgId);
                    await scratchOrgUtils_1.default.deleteScratchOrg(this.hubOrg, [
                        activeScratchOrgRecordId,
                    ]);
                    sfpowerkit_1.SFPowerkit.log(`Succesfully deleted scratchorg  ${scratchOrg.username}`, core_1.LoggerLevel.TRACE);
                }
                catch (error) {
                    sfpowerkit_1.SFPowerkit.log(`Unable to delete the scratchorg ${scratchOrg.username}..`, core_1.LoggerLevel.WARN);
                }
                failed++;
            }
        }
        return { success: success, failed: failed };
    }
    allocateScratchOrgsPerTag(remainingScratchOrgs, countOfActiveScratchOrgs, tag, poolUser) {
        sfpowerkit_1.SFPowerkit.log("Remaining ScratchOrgs" + remainingScratchOrgs, core_1.LoggerLevel.TRACE);
        poolUser.current_allocation = countOfActiveScratchOrgs;
        poolUser.to_allocate = 0;
        poolUser.to_satisfy_max =
            poolUser.max_allocation - poolUser.current_allocation > 0
                ? poolUser.max_allocation - poolUser.current_allocation
                : 0;
        if (poolUser.to_satisfy_max > 0 &&
            poolUser.to_satisfy_max <= remainingScratchOrgs) {
            poolUser.to_allocate = poolUser.to_satisfy_max;
        }
        else if (poolUser.to_satisfy_max > 0 &&
            poolUser.to_satisfy_max > remainingScratchOrgs) {
            poolUser.to_allocate = remainingScratchOrgs;
        }
        sfpowerkit_1.SFPowerkit.log("Computed Allocation" + JSON.stringify(poolUser), core_1.LoggerLevel.TRACE);
        return poolUser.to_allocate;
    }
    allocateScratchOrgsPerUser(remainingScratchOrgs, scratchOrgsRecordAsMapByUser, poolUsers) {
        let totalToBeAllocated = 0;
        //sort pooleconfig.poolusers based on priority
        poolUsers = poolUsers.sort((a, b) => a.priority - b.priority);
        let totalMaxOrgRequired = 0, totalMinOrgRequired = 0;
        poolUsers.forEach((pooluser) => {
            sfpowerkit_1.SFPowerkit.log(pooluser, core_1.LoggerLevel.TRACE);
            pooluser.to_allocate = 0;
            if (scratchOrgsRecordAsMapByUser[pooluser.username]) {
                pooluser.current_allocation =
                    scratchOrgsRecordAsMapByUser[pooluser.username];
                pooluser.to_satisfy_max =
                    pooluser.max_allocation - pooluser.current_allocation > 0
                        ? pooluser.max_allocation - pooluser.current_allocation
                        : 0;
                pooluser.to_satisfy_min =
                    pooluser.min_allocation - pooluser.current_allocation > 0
                        ? pooluser.min_allocation - pooluser.current_allocation
                        : 0;
            }
            else {
                pooluser.current_allocation = 0;
                pooluser.to_satisfy_max = pooluser.max_allocation;
                pooluser.to_satisfy_min = pooluser.min_allocation;
            }
            totalMaxOrgRequired += pooluser.to_satisfy_max;
            totalMinOrgRequired += pooluser.to_satisfy_min;
        });
        //All good..
        if (totalMaxOrgRequired <= remainingScratchOrgs) {
            // Satisfy max. allocate max
            poolUsers.forEach((pooluser) => {
                pooluser.to_allocate = pooluser.to_satisfy_max;
                totalToBeAllocated += pooluser.to_satisfy_max;
            });
        }
        else if (totalMinOrgRequired <= remainingScratchOrgs) {
            // Satisfy min
            //First allocate minimum to everyone
            poolUsers.forEach((pooluser) => {
                pooluser.to_allocate = pooluser.to_satisfy_min;
                totalToBeAllocated += pooluser.to_satisfy_min;
            });
            //Check for left overs
            let leftOver = remainingScratchOrgs - totalMinOrgRequired;
            if (leftOver > 0) {
                //Allocate LeftOver in a round robin model
                while (leftOver > 0) {
                    poolUsers.forEach((pooluser) => {
                        if (leftOver == 0)
                            return;
                        if (pooluser.current_allocation + pooluser.to_allocate <
                            pooluser.to_satisfy_max) {
                            pooluser.to_allocate++;
                            totalToBeAllocated++;
                            leftOver--;
                        }
                    });
                }
            }
        }
        else {
            let leftOver = remainingScratchOrgs;
            //Allocate LeftOver in a round robin model
            while (leftOver >= 0) {
                poolUsers.forEach((pooluser) => {
                    if (pooluser.current_allocation + pooluser.to_allocate <
                        pooluser.to_satisfy_max) {
                        pooluser.to_allocate++;
                        totalToBeAllocated++;
                        leftOver--;
                    }
                });
            }
        }
        return totalToBeAllocated;
    }
    async ipRangeRelaxer(scratchOrg) {
        //executue using bash
        sfpowerkit_1.SFPowerkit.log(`Relaxing ip ranges for scratchOrg with user ${scratchOrg.username}`, core_1.LoggerLevel.INFO);
        const connection = await core_1.Connection.create({
            authInfo: await core_1.AuthInfo.create({ username: scratchOrg.username }),
        });
        if (this.poolConfig.pool.relax_all_ip_ranges) {
            this.poolConfig.pool.relax_ip_ranges = [];
            return relaxIPRangeImpl_1.default.setIp(connection, scratchOrg.username, this.poolConfig.pool.relax_ip_ranges, this.poolConfig.pool.relax_all_ip_ranges);
        }
        else {
            return relaxIPRangeImpl_1.default.setIp(connection, scratchOrg.username, this.poolConfig.pool.relax_ip_ranges);
        }
    }
    async scriptExecutor(scriptFilePath, scratchOrg, hubOrgUserName) {
        //executue using bash
        let cmd;
        sfpowerkit_1.SFPowerkit.log(`Script File Path: ${scriptFilePath}`, core_1.LoggerLevel.TRACE);
        scriptFilePath = path.normalize(scriptFilePath);
        if (process.platform != "win32") {
            cmd = `bash ${scriptFilePath}  ${scratchOrg.username}  ${hubOrgUserName} `;
        }
        else {
            cmd = `cmd.exe /c ${scriptFilePath}  ${scratchOrg.username}  ${hubOrgUserName}`;
        }
        sfpowerkit_1.SFPowerkit.log(`Executing command: ${cmd}`, core_1.LoggerLevel.INFO);
        sfpowerkit_1.SFPowerkit.log(`Executing script for ${scratchOrg.alias} with username: ${scratchOrg.username}`, core_1.LoggerLevel.INFO);
        sfpowerkit_1.SFPowerkit.log(`Script Execution result is being written to script_exec_outputs/${scratchOrg.alias}.log, Please note this will take a significant time depending on the  script being executed`, core_1.LoggerLevel.INFO);
        let fdr = fs.openSync(`script_exec_outputs/${scratchOrg.alias}.log`, "a");
        return new Promise((resolve, reject) => {
            let ls = child_process_1.exec(cmd, { cwd: process.cwd() }, (error, stdout, stderr) => {
                if (error) {
                    sfpowerkit_1.SFPowerkit.log(`Failed to execute script for ${scratchOrg.username}`, core_1.LoggerLevel.WARN);
                    scratchOrg.isScriptExecuted = false;
                    resolve({
                        isSuccess: false,
                        message: error.message,
                        scratchOrgUsername: scratchOrg.username,
                        status: "failure",
                    });
                    return;
                }
                scratchOrg.isScriptExecuted = true;
                if ((this.poolConfig.pool.relax_all_ip_ranges ||
                    this.poolConfig.pool.relax_ip_ranges) &&
                    !this.ipRangeExecResultsAsObject[scratchOrg.username]["success"])
                    scratchOrg.isScriptExecuted = false;
                if (scratchOrg.isScriptExecuted) {
                    sfpowerkit_1.SFPowerkit.log(`Script Execution completed for ${scratchOrg.username} with alias ${scratchOrg.alias}`, core_1.LoggerLevel.INFO);
                    scratchOrgUtils_1.default.setScratchOrgInfo({
                        Id: scratchOrg.recordId,
                        Pooltag__c: this.poolConfig.pool.tag,
                        Allocation_status__c: scratchOrgUtils_1.default.isNewVersionCompatible
                            ? "Available"
                            : "",
                        Password__c: scratchOrg.password,
                    }, this.hubOrg).then((result) => {
                        scratchOrg.isScriptExecuted = true;
                        fs.closeSync(fdr);
                        resolve({
                            isSuccess: true,
                            message: "Successfuly set the scratch org record in Pool",
                            scratchOrgUsername: scratchOrg.username,
                            status: "success",
                        });
                    }, (reason) => {
                        fs.closeSync(fdr);
                        scratchOrg.isScriptExecuted = false;
                        resolve({
                            isSuccess: false,
                            message: "Unable to set the scratch org record in Pool",
                            scratchOrgUsername: scratchOrg.username,
                            status: "failure",
                        });
                    });
                }
            });
            ls.stderr.on("data", function (data) {
                fs.appendFileSync(`script_exec_outputs/${scratchOrg.alias}.log`, data);
            });
            ls.stdout.on("data", function (data) {
                fs.appendFileSync(`script_exec_outputs/${scratchOrg.alias}.log`, data);
            });
        });
    }
}
exports.default = PoolCreateImpl;
//# sourceMappingURL=poolCreateImpl.js.map