"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const sfpowerkit_1 = require("../../../sfpowerkit");
const scratchOrgUtils_1 = __importDefault(require("../../../utils/scratchOrgUtils"));
const getUserDetails_1 = require("../../../utils/getUserDetails");
class PoolFetchImpl {
    constructor(hubOrg, tag, mypool, sendToUser) {
        this.hubOrg = hubOrg;
        this.tag = tag;
        this.mypool = mypool;
        this.sendToUser = sendToUser;
    }
    async execute() {
        await scratchOrgUtils_1.default.checkForNewVersionCompatible(this.hubOrg);
        const results = (await scratchOrgUtils_1.default.getScratchOrgsByTag(this.tag, this.hubOrg, this.mypool, true));
        let emaiId;
        if (this.sendToUser) {
            try {
                emaiId = await getUserDetails_1.getUserEmail(this.sendToUser, this.hubOrg);
            }
            catch (error) {
                sfpowerkit_1.SFPowerkit.log("Unable to fetch details of the specified user, Check whether the user exists in the org ", core_1.LoggerLevel.ERROR);
                throw new core_1.SfdxError("Failed to fetch user details");
            }
        }
        let soDetail;
        if (results.records.length > 0) {
            sfpowerkit_1.SFPowerkit.log(`${this.tag} pool has ${results.records.length} Scratch orgs available`, core_1.LoggerLevel.TRACE);
            for (let element of results.records) {
                let allocateSO = await scratchOrgUtils_1.default.setScratchOrgInfo({ Id: element.Id, Allocation_status__c: "Allocate" }, this.hubOrg);
                if (allocateSO === true) {
                    sfpowerkit_1.SFPowerkit.log(`Scratch org ${element.SignupUsername} is allocated from the pool. Expiry date is ${element.ExpirationDate}`, core_1.LoggerLevel.TRACE);
                    soDetail = {};
                    soDetail["Id"] = element.Id;
                    soDetail.orgId = element.ScratchOrg;
                    soDetail.loginURL = element.LoginUrl;
                    soDetail.username = element.SignupUsername;
                    soDetail.password = element.Password__c;
                    soDetail.expityDate = element.ExpirationDate;
                    soDetail.status = "Assigned";
                    break;
                }
                else {
                    sfpowerkit_1.SFPowerkit.log(`Scratch org ${element.SignupUsername} allocation failed. trying to get another Scratch org from ${this.tag} pool`, core_1.LoggerLevel.TRACE);
                }
            }
        }
        if (results.records.length == 0 || !soDetail) {
            throw new core_1.SfdxError(`No scratch org available at the moment for ${this.tag}, try again in sometime.`);
        }
        if (this.sendToUser) {
            //Fetch the email for user id
            try {
                //Send an email for username
                await scratchOrgUtils_1.default.shareScratchOrgThroughEmail(emaiId, soDetail, this.hubOrg);
            }
            catch (error) {
                sfpowerkit_1.SFPowerkit.log("Unable to send the scratchorg details to specified user. Check whether the user exists in the org", core_1.LoggerLevel.ERROR);
            }
        }
        return soDetail;
    }
}
exports.default = PoolFetchImpl;
//# sourceMappingURL=poolFetchImpl.js.map