"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const DiffGenerators = __importStar(require("./diffgenerators/export"));
const util_1 = require("util");
// Gets the xml files and passes them into diff generators
// Output : csv or json format
class DataModelSourceDiffImpl {
    constructor(git, baseline, target, packageDirectories) {
        this.git = git;
        this.baseline = baseline;
        this.target = target;
        this.packageDirectories = packageDirectories;
        this.diffGenerators = {
            customfield: DiffGenerators.SourceDiffGenerator,
            recordtype: DiffGenerators.SourceDiffGenerator,
            businessprocess: DiffGenerators.SourceDiffGenerator
        };
        this.filePattern = {
            customfield: "field",
            recordtype: "recordType",
            businessprocess: "businessProcess"
        };
    }
    async exec() {
        const sourceDiffResult = [];
        for (let metadataType in this.diffGenerators) {
            let changedFiles = await this.getNameOfChangedFiles(this.git, this.baseline, this.target, metadataType);
            if (!util_1.isNullOrUndefined(this.packageDirectories)) {
                changedFiles = this.filterByPackageDirectory(changedFiles, this.packageDirectories);
            }
            let sourceDiffGenerator = new this.diffGenerators[metadataType](this.baseline, this.target);
            for (let file of changedFiles) {
                let fileRevFrom = await this.git
                    .show([`${this.baseline}:${file}`])
                    .catch(err => { });
                let fileRevTo = await this.git
                    .show([`${this.target}:${file}`])
                    .catch(err => { });
                let diff = await sourceDiffGenerator.compareRevisions(fileRevFrom, fileRevTo, file);
                // Aggregate individual file diffs in the source diff result
                if (diff) {
                    sourceDiffResult.push(diff);
                }
            }
        }
        return sourceDiffResult;
    }
    async getNameOfChangedFiles(git, baseline, target, metadataType) {
        let gitDiffResult = await git.diff([
            baseline,
            target,
            "--name-only",
            "--",
            `**/objects/**/*${this.filePattern[metadataType]}-meta.xml`
        ]);
        let changedFiles = gitDiffResult.split("\n");
        changedFiles.pop();
        return changedFiles;
    }
    filterByPackageDirectory(changedFiles, packageDirectories) {
        let filteredChangedFiles = changedFiles.filter(file => {
            let isFileInPackageDir;
            packageDirectories.forEach(dir => {
                if (file.includes(dir))
                    isFileInPackageDir = true;
            });
            return isFileInPackageDir;
        });
        return filteredChangedFiles;
    }
}
exports.default = DataModelSourceDiffImpl;
//# sourceMappingURL=DataModelSourceDiffImpl.js.map