"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildCommands = void 0;
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
const pascalCase = (it) => it
    .map((word) => word[0].toUpperCase() + word.slice(1).toLowerCase())
    .join("");
function preprocessCommandsDir(commandsDir, namespace, parts) {
    const cmdArray = [];
    const dir = path.join(commandsDir, ...parts);
    fs.readdirSync(dir)
        .sort((a, b) => {
        const statA = fs.statSync(path.join(dir, a));
        const statB = fs.statSync(path.join(dir, b));
        return statA.isFile() === statB.isFile()
            ? 0
            : statA.isFile() <= statB.isFile()
                ? 1
                : -1;
    })
        .forEach((fileOrDir) => {
        const commandFile = path.join(dir, fileOrDir);
        const fileNameWithoutExt = fileOrDir.replace(".js", "");
        const newParts = [...parts, fileNameWithoutExt];
        const stat = fs.statSync(commandFile);
        if (stat.isFile()) {
            if (newParts.length > 0 && path.extname(commandFile) === ".js") {
                cmdArray.push({
                    commandFile,
                    commandId: [namespace, ...newParts].join(":"),
                    commandName: pascalCase([...newParts, "command"]),
                });
            }
        }
        else if (stat.isDirectory()) {
            cmdArray.push(...preprocessCommandsDir(commandsDir, namespace, newParts));
        }
    });
    return cmdArray;
}
function processBaseCommand(moduleDir, namespace) {
    if (fs.existsSync(path.join(moduleDir, `${namespace}.js`))) {
        return {
            commandFile: path.join(moduleDir, `${namespace}.js`),
            commandId: namespace,
            commandName: pascalCase([namespace, "command"]),
        };
    }
}
function buildCommands(createCommand, moduleDir, namespace) {
    const base = processBaseCommand(moduleDir, namespace);
    const nsApi = base
        ? createCommand(base.commandId, base.commandName, base.commandFile)
        : {};
    preprocessCommandsDir(path.join(moduleDir, namespace), namespace, []).forEach(({ commandId, commandName, commandFile }) => {
        const parts = commandId.split(":").slice(1);
        parts.reduce((api, part) => {
            if (!api.hasOwnProperty(part)) {
                if (parts[parts.length - 1] === part) {
                    api[part] = createCommand(commandId, commandName, commandFile);
                }
                else {
                    api[part] = {};
                }
            }
            return api[part];
        }, nsApi);
    });
    return nsApi;
}
exports.buildCommands = buildCommands;
//# sourceMappingURL=builder.js.map