"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCommand = void 0;
const buildArgs_1 = require("./buildArgs");
const hookStd = require("hook-std");
const process_errors_1 = require("./process-errors");
const realStdoutWrite = process.stdout.write;
const realStderrWrite = process.stderr.write;
let sfdxErrors = [];
// @ts-ignore
process.on("cmdError", errObj => {
    sfdxErrors.push(errObj);
});
const unhookStd = () => {
    process.stdout.write = realStdoutWrite;
    process.stderr.write = realStderrWrite;
};
exports.createCommand = (commandId, commandName, commandFile) => (flags = {}, opts = []) => new Promise((resolve, reject) => {
    // tslint:disable-next-line:non-literal-require
    const required = require(commandFile);
    const command = required.default || required[commandName];
    command.id = commandId;
    const args = buildArgs_1.buildArgs(flags, opts);
    const quiet = Boolean(flags.quiet) || false;
    let currentHookFlag = false;
    if (quiet) {
        hookStd(() => undefined);
        currentHookFlag = true;
    }
    sfdxErrors = [];
    command
        .run(args)
        .then(sfdxResult => {
        if (quiet && currentHookFlag) {
            currentHookFlag = false;
            unhookStd();
        }
        if (sfdxErrors.length) {
            throw sfdxErrors;
        }
        if (process.exitCode) {
            process.exitCode = 0;
        }
        resolve(sfdxResult);
    })
        .catch(sfdxErr => {
        if (quiet && currentHookFlag) {
            currentHookFlag = false;
            unhookStd();
        }
        if (process.exitCode) {
            process.exitCode = 0;
        }
        reject(process_errors_1.parseErrors(sfdxErr));
    });
});
//# sourceMappingURL=run.js.map