"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const retry = require("async-retry");
class QueryExecutor {
    constructor(conn) {
        this.conn = conn;
    }
    async executeQuery(query, tooling) {
        let results;
        if (tooling) {
            results = await retry(async (bail) => {
                try {
                    return (await this.conn.tooling.query(query));
                }
                catch (error) {
                    throw error;
                }
            }, { retries: 3, minTimeout: 2000 });
        }
        else {
            results = await retry(async (bail) => {
                try {
                    return (await this.conn.query(query));
                }
                catch (error) {
                    throw error;
                }
            }, { retries: 3, minTimeout: 2000 });
        }
        if (!results.done) {
            let tempRecords = results.records;
            while (!results.done) {
                results = await this.queryMore(results.nextRecordsUrl, tooling);
                tempRecords = tempRecords.concat(results.records);
            }
            results.records = tempRecords;
        }
        return results.records;
    }
    async queryMore(url, tooling) {
        let result;
        if (tooling) {
            result = await retry(async (bail) => {
                try {
                    return (await this.conn.tooling.queryMore(url));
                }
                catch (error) {
                    throw error;
                }
            }, { retries: 3, minTimeout: 2000 });
        }
        else {
            result = await retry(async (bail) => {
                try {
                    return (await this.conn.tooling.query(url));
                }
                catch (error) {
                    throw error;
                }
            }, { retries: 3, minTimeout: 2000 });
        }
        return result;
    }
}
exports.default = QueryExecutor;
//# sourceMappingURL=queryExecutor.js.map